import logging

# from fk.batch.BatchProcessor import BatchProcessor

logger = logging.getLogger(__name__)


def _test_batch_processor():
    config = {}
    bp = BatchProcessor(config)
    return True
